@@ -1,4 +1,5 @@
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE RankNTypes #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
@@ -19,8 +20,7 @@ module Yesod.Persist.Core
     ) where
 
 import Database.Persist
-import Database.Persist.Sql (SqlPersistT, unSqlPersistT)
-import Control.Monad.Trans.Reader (runReaderT)
+import Control.Monad.Trans.Reader (ReaderT, runReaderT)
 
 import Yesod.Core
 import Data.Conduit
@@ -31,11 +31,14 @@ import Control.Exception (throwIO)
 import Yesod.Core.Types (HandlerContents (HCError))
 import qualified Database.Persist.Sql as SQL
 
-type YesodDB site = YesodPersistBackend site (HandlerT site IO)
+unSqlPersistT :: a -> a
+unSqlPersistT = id
 
-class Monad (YesodPersistBackend site (HandlerT site IO)) => YesodPersist site where
-    type YesodPersistBackend site :: (* -> *) -> * -> *
-    runDB :: YesodPersistBackend site (HandlerT site IO) a -> HandlerT site IO a
+type YesodDB site = ReaderT (YesodPersistBackend site) (HandlerT site IO)
+
+class Monad (YesodDB site) => YesodPersist site where
+    type YesodPersistBackend site
+    runDB :: YesodDB site a -> HandlerT site IO a
 
 -- | Helper for creating 'runDB'.
 --
@@ -71,13 +74,13 @@ class YesodPersist site => YesodPersistRunner site where
     getDBRunner :: HandlerT site IO (DBRunner site, HandlerT site IO ())
 
 newtype DBRunner site = DBRunner
-    { runDBRunner :: forall a. YesodPersistBackend site (HandlerT site IO) a -> HandlerT site IO a
+    { runDBRunner :: forall a. YesodDB site a -> HandlerT site IO a
     }
 
 -- | Helper for implementing 'getDBRunner'.
 --
 -- Since 1.2.0
-defaultGetDBRunner :: YesodPersistBackend site ~ SqlPersistT
+defaultGetDBRunner :: YesodPersistBackend site ~ SQL.SqlBackend
                    => (site -> Pool SQL.Connection)
                    -> HandlerT site IO (DBRunner site, HandlerT site IO ())
 defaultGetDBRunner getPool = do
@@ -106,7 +109,7 @@ defaultGetDBRunner getPool = do
 --
 -- Since 1.2.0
 runDBSource :: YesodPersistRunner site
-            => Source (YesodPersistBackend site (HandlerT site IO)) a
+            => Source (YesodDB site) a
             -> Source (HandlerT site IO) a
 runDBSource src = do
     (dbrunner, cleanup) <- lift getDBRunner
@@ -116,19 +119,14 @@ runDBSource src = do
 -- | Extends 'respondSource' to create a streaming database response body.
 respondSourceDB :: YesodPersistRunner site
                 => ContentType
-                -> Source (YesodPersistBackend site (HandlerT site IO)) (Flush Builder)
+                -> Source (YesodDB site) (Flush Builder)
                 -> HandlerT site IO TypedContent
 respondSourceDB ctype = respondSource ctype . runDBSource
 
 -- | Get the given entity by ID, or return a 404 not found if it doesn't exist.
-get404 :: ( PersistStore (t m)
-          , PersistEntity val
-          , Monad (t m)
-          , m ~ HandlerT site IO
-          , MonadTrans t
-          , PersistMonadBackend (t m) ~ PersistEntityBackend val
-          )
-       => Key val -> t m val
+get404 :: (MonadIO m, PersistStore (PersistEntityBackend val), PersistEntity val)
+       => Key val
+       -> ReaderT (PersistEntityBackend val) m val
 get404 key = do
     mres <- get key
     case mres of
@@ -137,14 +135,9 @@ get404 key = do
 
 -- | Get the given entity by unique key, or return a 404 not found if it doesn't
 --   exist.
-getBy404 :: ( PersistUnique (t m)
-            , PersistEntity val
-            , m ~ HandlerT site IO
-            , Monad (t m)
-            , MonadTrans t
-            , PersistEntityBackend val ~ PersistMonadBackend (t m)
-            )
-         => Unique val -> t m (Entity val)
+getBy404 :: (PersistUnique (PersistEntityBackend val), PersistEntity val, MonadIO m)
+         => Unique val
+         -> ReaderT (PersistEntityBackend val) m (Entity val)
 getBy404 key = do
     mres <- getBy key
     case mres of
@@ -155,9 +148,3 @@ getBy404 key = do
 -- GHC 7.4.2 that leads to segfaults. This is a workaround.
 notFound' :: MonadIO m => m a
 notFound' = liftIO $ throwIO $ HCError NotFound
-
-instance MonadHandler m => MonadHandler (SqlPersistT m) where
-    type HandlerSite (SqlPersistT m) = HandlerSite m
-    liftHandlerT = lift . liftHandlerT
-instance MonadWidget m => MonadWidget (SqlPersistT m) where
-    liftWidgetT = lift . liftWidgetT
